@@ -32,6 +32,8 @@
 #include <config.h>
 #endif
 
+#include <string.h>
+
 #include "na-gnome-vfs-uri.h"
 #include "na-selected-info.h"
 
@@ -50,6 +52,10 @@ struct NASelectedInfoPrivate {
 	GFile         *location;
 	gchar         *mimetype;
 	GFileType      file_type;
+	gboolean       can_read;
+	gboolean       can_write;
+	gboolean       can_execute;
+	gchar         *owner;
 };
 
 
@@ -176,6 +182,7 @@ instance_finalize( GObject *object )
 
 	g_free( self->private->uri );
 	g_free( self->private->mimetype );
+	g_free( self->private->owner );
 
 	g_free( self->private );
 
@@ -249,15 +256,15 @@ na_selected_info_copy_list( GList *files )
 
 /**
  * na_selected_info_free_list:
- * @list: a #GList of #NASelectedInfo items.
+ * @files: a #GList of #NASelectedInfo items.
  *
- * Frees up the #GList @list.
+ * Frees up the #GList @files.
  */
 void
-na_selected_info_free_list( GList *list )
+na_selected_info_free_list( GList *files )
 {
-	g_list_foreach( list, ( GFunc ) g_object_unref, NULL );
-	g_list_free( list );
+	g_list_foreach( files, ( GFunc ) g_object_unref, NULL );
+	g_list_free( files );
 }
 
 /**
@@ -285,6 +292,56 @@ na_selected_info_get_location( const NASelectedInfo *nsi )
 	return( location );
 }
 
+/**
+ * na_selected_info_get_basename:
+ * @nsi: this #NASelectedInfo object.
+ *
+ * Returns: the basename of the file associated with this
+ * #NASelectedInfo object, as a newly allocated string which
+ * must be g_free() by the caller.
+ */
+gchar *
+na_selected_info_get_basename( const NASelectedInfo *nsi )
+{
+	gchar *basename;
+
+	g_return_val_if_fail( NA_IS_SELECTED_INFO( nsi ), NULL );
+
+	basename = NULL;
+
+	if( !nsi->private->dispose_has_run ){
+
+		basename = g_strdup( g_path_get_basename( nsi->private->vfs->path ));
+	}
+
+	return( basename );
+}
+
+/**
+ * na_selected_info_get_dirname:
+ * @nsi: this #NASelectedInfo object.
+ *
+ * Returns: the dirname of the file associated with this
+ * #NASelectedInfo object, as a newly allocated string which
+ * must be g_free() by the caller.
+ */
+gchar *
+na_selected_info_get_dirname( const NASelectedInfo *nsi )
+{
+	gchar *dirname;
+
+	g_return_val_if_fail( NA_IS_SELECTED_INFO( nsi ), NULL );
+
+	dirname = NULL;
+
+	if( !nsi->private->dispose_has_run ){
+
+		dirname = g_strdup( g_path_get_dirname( nsi->private->vfs->path ));
+	}
+
+	return( dirname );
+}
+
 /**
  * na_selected_info_get_mime_type:
  * @nsi: this #NASelectedInfo object.
@@ -403,6 +460,125 @@ na_selected_info_is_directory( const NASelectedInfo *nsi )
 	return( is_dir );
 }
 
+/**
+ * na_selected_info_is_executable:
+ * @nsi: this #NASelectedInfo object.
+ *
+ * Returns: %TRUE if the item is executable by the user, %FALSE else.
+ */
+gboolean
+na_selected_info_is_executable( const NASelectedInfo *nsi )
+{
+	gboolean is_exe;
+
+	g_return_val_if_fail( NA_IS_SELECTED_INFO( nsi ), FALSE );
+
+	is_exe = FALSE;
+
+	if( !nsi->private->dispose_has_run ){
+
+		is_exe = nsi->private->can_execute;
+	}
+
+	return( is_exe );
+}
+
+/**
+ * na_selected_info_is_local:
+ * @nsi: this #NASelectedInfo object.
+ *
+ * Returns: %TRUE if the item is on a local filesystem, %FALSE else.
+ */
+gboolean
+na_selected_info_is_local( const NASelectedInfo *nsi )
+{
+	gboolean is_local;
+	gchar *scheme;
+
+	g_return_val_if_fail( NA_IS_SELECTED_INFO( nsi ), FALSE );
+
+	is_local = FALSE;
+
+	if( !nsi->private->dispose_has_run ){
+
+		scheme = na_selected_info_get_uri_scheme( nsi );
+		is_local = ( strcmp( scheme, "file" ) == 0 );
+		g_free( scheme );
+	}
+
+	return( is_local );
+}
+
+/**
+ * na_selected_info_is_owner:
+ * @nsi: this #NASelectedInfo object.
+ * @user: the user to be tested against the owner of the @nsi object.
+ *
+ * Returns: %TRUE if the item is a owner, %FALSE else.
+ */
+gboolean
+na_selected_info_is_owner( const NASelectedInfo *nsi, const gchar *user )
+{
+	gboolean is_owner;
+
+	g_return_val_if_fail( NA_IS_SELECTED_INFO( nsi ), FALSE );
+
+	is_owner = FALSE;
+
+	if( !nsi->private->dispose_has_run ){
+
+		is_owner = ( strcmp( nsi->private->owner, user ) == 0 );
+	}
+
+	return( is_owner );
+}
+
+/**
+ * na_selected_info_is_readable:
+ * @nsi: this #NASelectedInfo object.
+ *
+ * Returns: %TRUE if the item is a readable, %FALSE else.
+ */
+gboolean
+na_selected_info_is_readable( const NASelectedInfo *nsi )
+{
+	gboolean is_readable;
+
+	g_return_val_if_fail( NA_IS_SELECTED_INFO( nsi ), FALSE );
+
+	is_readable = FALSE;
+
+	if( !nsi->private->dispose_has_run ){
+
+		is_readable = nsi->private->can_read;
+	}
+
+	return( is_readable );
+}
+
+/**
+ * na_selected_info_is_writable:
+ * @nsi: this #NASelectedInfo object.
+ *
+ * Returns: %TRUE if the item is a writable, %FALSE else.
+ */
+gboolean
+na_selected_info_is_writable( const NASelectedInfo *nsi )
+{
+	gboolean is_writable;
+
+	g_return_val_if_fail( NA_IS_SELECTED_INFO( nsi ), FALSE );
+
+	is_writable = FALSE;
+
+	if( !nsi->private->dispose_has_run ){
+
+		is_writable = nsi->private->can_write;
+	}
+
+	return( is_writable );
+}
+
 /**
  * na_selected_info_create_for_uri:
  * @uri: an URI.
@@ -475,7 +651,12 @@ query_file_attributes( NASelectedInfo *nsi )
 
 	error = NULL;
 	GFileInfo *info = g_file_query_info( nsi->private->location,
-			G_FILE_ATTRIBUTE_STANDARD_TYPE "," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+			G_FILE_ATTRIBUTE_STANDARD_TYPE
+				"," G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE
+				"," G_FILE_ATTRIBUTE_ACCESS_CAN_READ
+				"," G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE
+				"," G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE
+				"," G_FILE_ATTRIBUTE_OWNER_USER,
 			G_FILE_QUERY_INFO_NONE, NULL, &error );
 
 	if( error ){
@@ -490,5 +671,11 @@ query_file_attributes( NASelectedInfo *nsi )
 
 	nsi->private->file_type = ( GFileType ) g_file_info_get_attribute_uint32( info, G_FILE_ATTRIBUTE_STANDARD_TYPE );
 
+	nsi->private->can_read = g_file_info_get_attribute_boolean( info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ );
+	nsi->private->can_write = g_file_info_get_attribute_boolean( info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE );
+	nsi->private->can_execute = g_file_info_get_attribute_boolean( info, G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE );
+
+	nsi->private->owner = g_strdup( g_file_info_get_attribute_as_string( info, G_FILE_ATTRIBUTE_OWNER_USER ));
+
 	g_object_unref( info );
 }
